@@ -38,6 +38,7 @@
 #include "gtksourceview-marshal.h"
 #include "gtksourceiter.h"
 #include "gtksourcestylemanager.h"
+#include "gtksourcestyle-private.h"
 
 /*
 #define ENABLE_DEBUG
@@ -176,7 +177,7 @@ gtk_source_buffer_class_init (GtkSourceBufferClass *klass)
 							       _("Highlight"),
 							       _("Whether to highlight syntax "
 								 "in the buffer"),
-							       FALSE,
+							       TRUE,
 							       G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class,
@@ -276,6 +277,11 @@ gtk_source_buffer_init (GtkSourceBuffer *buffer)
 
 	priv->markers = g_array_new (FALSE, FALSE, sizeof (GtkSourceMarker *));
 
+	priv->highlight = TRUE;
+	priv->style_scheme = _gtk_source_style_scheme_get_default ();
+	if (priv->style_scheme != NULL)
+		g_object_ref (priv->style_scheme);
+
 	g_signal_connect (G_OBJECT (buffer),
 			  "mark_set",
 			  G_CALLBACK (gtk_source_buffer_move_cursor),
@@ -306,9 +312,6 @@ gtk_source_buffer_finalize (GObject *object)
 	if (buffer->priv->markers)
 		g_array_free (buffer->priv->markers, TRUE);
 
-	if (buffer->priv->style_scheme)
-		g_object_unref (buffer->priv->style_scheme);
-
 	G_OBJECT_CLASS (gtk_source_buffer_parent_class)->finalize (object);
 }
 
@@ -342,6 +345,12 @@ gtk_source_buffer_dispose (GObject *object)
 		buffer->priv->language = NULL;
 	}
 
+	if (buffer->priv->style_scheme != NULL)
+	{
+		g_object_unref (buffer->priv->style_scheme);
+		buffer->priv->style_scheme = NULL;
+	}
+
 	G_OBJECT_CLASS (gtk_source_buffer_parent_class)->dispose (object);
 }
 
@@ -1065,7 +1074,8 @@ update_bracket_match_style (GtkSourceBuffer *buffer)
 		if (buffer->priv->style_scheme)
 			style = gtk_source_style_scheme_get_matching_brackets_style (buffer->priv->style_scheme);
 		_gtk_source_style_apply (style, buffer->priv->bracket_match_tag);
-		gtk_source_style_free (style);
+		if (style != NULL)
+			g_object_unref (style);
 	}
 }
 
