@@ -3,36 +3,37 @@
  *
  *  Copyright (C) 2003 - Paolo Maggi <paolo.maggi@polito.it>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Library General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2.1 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
- *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
+
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "gtksourceview-i18n.h"
-#include "gtksourcestylescheme.h"
+#include "gtksourcestylemanager.h"
 #include "gtksourceview.h"
 #include "gtksourcelanguage-private.h"
+#include "gtksourcestyle-private.h"
+#include <libxml/parser.h>
 #include <string.h>
 
-#define STYLE_HAS_FOREGROUND(s) ((s) && ((s)->mask & GTK_SOURCE_STYLE_USE_FOREGROUND))
-#define STYLE_HAS_BACKGROUND(s) ((s) && ((s)->mask & GTK_SOURCE_STYLE_USE_BACKGROUND))
-
 #define STYLE_TEXT		"text"
 #define STYLE_SELECTED		"text-selected"
 #define STYLE_BRACKET_MATCH	"bracket-match"
 #define STYLE_BRACKET_MISMATCH	"bracket-mismatch"
 #define STYLE_CURSOR		"cursor"
+#define STYLE_SECONDARY_CURSOR	"secondary-cursor"
 #define STYLE_CURRENT_LINE	"current-line"
+#define STYLE_LINE_NUMBERS	"line-numbers"
 
 enum {
 	PROP_0,
@@ -44,6 +45,8 @@ struct _GtkSourceStyleSchemePrivate
 {
 	gchar *id;
 	gchar *name;
+	gchar *author;
+	gchar *description;
 	GtkSourceStyleScheme *parent;
 	gchar *parent_id;
 	GHashTable *styles;
@@ -57,6 +60,8 @@ gtk_source_style_scheme_finalize (GObject *object)
 	GtkSourceStyleScheme *scheme = GTK_SOURCE_STYLE_SCHEME (object);
 
 	g_hash_table_destroy (scheme->priv->styles);
+	g_free (scheme->priv->author);
+	g_free (scheme->priv->description);
 	g_free (scheme->priv->id);
 	g_free (scheme->priv->name);
 	g_free (scheme->priv->parent_id);
@@ -170,7 +175,7 @@ gtk_source_style_scheme_init (GtkSourceStyleScheme *scheme)
 	scheme->priv = G_TYPE_INSTANCE_GET_PRIVATE (scheme, GTK_TYPE_SOURCE_STYLE_SCHEME,
 						    GtkSourceStyleSchemePrivate);
 	scheme->priv->styles = g_hash_table_new_full (g_str_hash, g_str_equal, g_free,
-						      (GDestroyNotify) gtk_source_style_free);
+						      (GDestroyNotify) g_object_unref);
 }
 
 /**
@@ -235,8 +240,8 @@ _gtk_source_style_scheme_new (const gchar *id,
  * @style_name: style name to find.
  *
  * Returns: style which corresponds to @style_name in the @scheme,
- * or %NULL when no style with this name found. Free it with
- * gtk_source_style_free().
+ * or %NULL when no style with this name found. Call g_object_unref()
+ * when you are done with it.
  *
  * Since: 2.0
  */
@@ -289,7 +294,7 @@ gtk_source_style_scheme_set_style (GtkSourceStyleScheme *scheme,
  * @scheme: a #GtkSourceStyleScheme.
  *
  * Returns: style which corresponds to "bracket-match" name, to use
- * in an editor. Free it with gtk_source_style_free().
+ * in an editor. Call g_object_unref() when you are done with it.
  *
  * Since: 2.0
  */
@@ -300,6 +305,43 @@ gtk_source_style_scheme_get_matching_brackets_style (GtkSourceStyleScheme *schem
 	return gtk_source_style_scheme_get_style (scheme, STYLE_BRACKET_MATCH);
 }
 
+static gboolean
+get_color (GtkSourceStyle *style,
+	   gboolean        foreground,
+	   GdkColor       *dest)
+{
+	const gchar *color;
+	guint mask;
+
+	if (style == NULL)
+		return FALSE;
+
+	if (foreground)
+	{
+		color = style->foreground;
+		mask = GTK_SOURCE_STYLE_USE_FOREGROUND;
+	}
+	else
+	{
+		color = style->background;
+		mask = GTK_SOURCE_STYLE_USE_BACKGROUND;
+	}
+
+	if (style->mask & mask)
+	{
+		if (color == NULL || !gdk_color_parse (color, dest))
+		{
+			g_warning ("%s: invalid color '%s'", G_STRLOC,
+				   color != NULL ? color : "(null)");
+			return FALSE;
+		}
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 /**
  * gtk_source_style_scheme_get_current_line_color:
  * @scheme: a #GtkSourceStyleScheme.
@@ -322,13 +364,11 @@ gtk_source_style_scheme_get_current_line_color (GtkSourceStyleScheme *scheme,
 
 	style = gtk_source_style_scheme_get_style (scheme, STYLE_CURRENT_LINE);
 
-	if (STYLE_HAS_FOREGROUND (style))
-	{
-		*color = style->foreground;
-		ret = TRUE;
-	}
+	ret = get_color (style, FALSE, color);
+
+	if (style != NULL)
+		g_object_unref (style);
 
-	gtk_source_style_free (style);
 	return ret;
 }
 
@@ -337,35 +377,122 @@ set_text_style (GtkWidget      *widget,
 		GtkSourceStyle *style,
 		GtkStateType    state)
 {
-	GdkColor *color;
+	GdkColor color;
+	GdkColor *color_ptr;
 
-	if (style != NULL && STYLE_HAS_BACKGROUND (style))
-		color = &style->background;
+	if (get_color (style, FALSE, &color))
+		color_ptr = &color;
 	else
-		color = NULL;
+		color_ptr = NULL;
 
-	gtk_widget_modify_base (widget, state, color);
+	gtk_widget_modify_base (widget, state, color_ptr);
 
-	if (style != NULL && STYLE_HAS_FOREGROUND (style))
-		color = &style->foreground;
+	if (get_color (style, TRUE, &color))
+		color_ptr = &color;
 	else
-		color = NULL;
+		color_ptr = NULL;
 
-	gtk_widget_modify_text (widget, state, color);
+	gtk_widget_modify_text (widget, state, color_ptr);
 }
 
 static void
-set_cursor_color (GtkWidget      *widget,
-		  GtkSourceStyle *style)
+set_line_numbers_style (GtkWidget      *widget,
+			GtkSourceStyle *style)
 {
-	GdkColor *color;
+	gint i;
+	GdkColor *fg_ptr = NULL;
+	GdkColor *bg_ptr = NULL;
+	GdkColor fg;
+	GdkColor bg;
 
-	if (style != NULL && STYLE_HAS_FOREGROUND (style))
-		color = &style->foreground;
+	if (get_color (style, TRUE, &fg))
+		fg_ptr = &fg;
+	if (get_color (style, FALSE, &bg))
+		bg_ptr = &bg;
+
+	for (i = 0; i < 5; ++i)
+	{
+		gtk_widget_modify_fg (widget, i, fg_ptr);
+		gtk_widget_modify_bg (widget, i, bg_ptr);
+	}
+}
+
+static void
+set_cursor_colors (GtkWidget      *widget,
+		   const GdkColor *primary,
+		   const GdkColor *secondary)
+{
+#if !GTK_CHECK_VERSION(2,11,3)
+	char *rc_string;
+	char *widget_name;
+
+	widget_name = g_strdup_printf ("gtk-source-view-%p", (gpointer) widget);
+
+	rc_string = g_strdup_printf (
+		"style \"%p\"\n"
+		"{\n"
+		"   GtkWidget::cursor-color = \"#%02x%02x%02x\"\n"
+		"   GtkWidget::secondary-cursor-color = \"#%02x%02x%02x\"\n"
+		"}\n"
+		"widget \"*.%s\" style \"%p\"\n",
+		(gpointer) widget,
+		primary->red >> 8, primary->green >> 8, primary->blue >> 8,
+		secondary->red >> 8, secondary->green >> 8, secondary->blue >> 8,
+		widget_name,
+		(gpointer) widget
+	);
+
+	gtk_rc_parse_string (rc_string);
+
+	if (strcmp (widget_name, gtk_widget_get_name (widget)) != 0)
+		gtk_widget_set_name (widget, widget_name);
+
+	g_free (rc_string);
+	g_free (widget_name);
+#else
+	gtk_widget_modify_cursor (widget, primary, secondary);
+#endif
+}
+
+static void
+unset_cursor_colors (GtkWidget *widget)
+{
+#if !GTK_CHECK_VERSION(2,11,3)
+	set_cursor_colors (widget,
+			   &widget->style->text[GTK_STATE_NORMAL],
+			   &widget->style->text_aa[GTK_STATE_NORMAL]);
+#else
+	gtk_widget_modify_cursor (widget, NULL, NULL);
+#endif
+}
+
+static void
+update_cursor_colors (GtkWidget      *widget,
+		      GtkSourceStyle *style_primary,
+		      GtkSourceStyle *style_secondary)
+{
+	GdkColor primary_color, secondary_color;
+	GdkColor *primary = NULL, *secondary = NULL;
+
+	if (get_color (style_primary, TRUE, &primary_color))
+		primary = &primary_color;
+
+	if (get_color (style_secondary, TRUE, &secondary_color))
+		secondary = &secondary_color;
+
+	if (primary != NULL && secondary == NULL)
+	{
+		secondary_color = widget->style->base[GTK_STATE_NORMAL];
+		secondary_color.red = ((gint) secondary_color.red + primary->red) / 2;
+		secondary_color.green = ((gint) secondary_color.green + primary->green) / 2;
+		secondary_color.blue = ((gint) secondary_color.blue + primary->blue) / 2;
+		secondary = &secondary_color;
+	}
+
+	if (primary != NULL)
+		set_cursor_colors (widget, primary, secondary);
 	else
-		color = &widget->style->text[GTK_STATE_NORMAL];
-
-	g_print ("implement me\n");
+		unset_cursor_colors (widget);
 }
 
 /**
@@ -386,7 +513,7 @@ _gtk_source_style_scheme_apply (GtkSourceStyleScheme *scheme,
 
 	if (scheme != NULL)
 	{
-		GtkSourceStyle *style;
+		GtkSourceStyle *style, *style2;
 
 		gtk_widget_ensure_style (widget);
 
@@ -395,15 +522,26 @@ _gtk_source_style_scheme_apply (GtkSourceStyleScheme *scheme,
 		set_text_style (widget, style, GTK_STATE_ACTIVE);
 		set_text_style (widget, style, GTK_STATE_PRELIGHT);
 		set_text_style (widget, style, GTK_STATE_INSENSITIVE);
-		gtk_source_style_free (style);
+		if (style != NULL)
+			g_object_unref (style);
 
 		style = gtk_source_style_scheme_get_style (scheme, STYLE_SELECTED);
 		set_text_style (widget, style, GTK_STATE_SELECTED);
-		gtk_source_style_free (style);
+		if (style != NULL)
+			g_object_unref (style);
+
+		style = gtk_source_style_scheme_get_style (scheme, STYLE_LINE_NUMBERS);
+		set_line_numbers_style (widget, style);
+		if (style != NULL)
+			g_object_unref (style);
 
 		style = gtk_source_style_scheme_get_style (scheme, STYLE_CURSOR);
-		set_cursor_color (widget, style);
-		gtk_source_style_free (style);
+		style2 = gtk_source_style_scheme_get_style (scheme, STYLE_SECONDARY_CURSOR);
+		update_cursor_colors (widget, style, style2);
+		if (style2 != NULL)
+			g_object_unref (style2);
+		if (style != NULL)
+			g_object_unref (style);
 	}
 	else
 	{
@@ -412,7 +550,8 @@ _gtk_source_style_scheme_apply (GtkSourceStyleScheme *scheme,
 		set_text_style (widget, NULL, GTK_STATE_PRELIGHT);
 		set_text_style (widget, NULL, GTK_STATE_INSENSITIVE);
 		set_text_style (widget, NULL, GTK_STATE_SELECTED);
-		set_cursor_color (widget, NULL);
+		set_line_numbers_style (widget, NULL);
+		unset_cursor_colors (widget);
 	}
 }
 
@@ -425,100 +564,85 @@ typedef struct {
 
 #define ERROR_QUARK (g_quark_from_static_string ("gtk-source-style-scheme-parser-error"))
 
-static gboolean
-str_to_bool (const gchar *string)
+static void
+get_bool (xmlNode    *node,
+	  const char *propname,
+	  guint      *mask,
+	  guint       mask_value,
+	  gboolean   *value)
 {
-	return !g_ascii_strcasecmp (string, "true") ||
-		!g_ascii_strcasecmp (string, "yes") ||
-		!g_ascii_strcasecmp (string, "1");
+	xmlChar *tmp = xmlGetProp (node, BAD_CAST propname);
+
+	if (tmp != NULL)
+	{
+		*mask |= mask_value;
+		*value = g_ascii_strcasecmp ((char*) tmp, "true") == 0 ||
+			 g_ascii_strcasecmp ((char*) tmp, "yes") == 0 ||
+			 g_ascii_strcasecmp ((char*) tmp, "1") == 0;
+	}
+
+	xmlFree (tmp);
 }
 
 static gboolean
 parse_style (GtkSourceStyleScheme *scheme,
-	     const gchar         **names,
-	     const gchar         **values,
+	     xmlNode              *node,
 	     gchar               **style_name_p,
 	     GtkSourceStyle      **style_p,
 	     GError              **error)
 {
 	GtkSourceStyle *use_style = NULL;
 	GtkSourceStyle *result = NULL;
-	const gchar *fg = NULL, *bg = NULL;
-	const gchar *style_name = NULL;
-	GtkSourceStyleMask mask = 0;
+	xmlChar *fg = NULL, *bg = NULL;
+	gchar *style_name = NULL;
+	guint mask = 0;
 	gboolean bold = FALSE;
 	gboolean italic = FALSE;
 	gboolean underline = FALSE;
 	gboolean strikethrough = FALSE;
+	xmlChar *tmp;
 
-	for ( ; names && *names; names++, values++)
+	tmp = xmlGetProp (node, BAD_CAST "name");
+	if (tmp == NULL)
 	{
-		const gchar *name = *names;
-		const gchar *val = *values;
+		g_set_error (error, ERROR_QUARK, 0, "name attribute missing");
+		return FALSE;
+	}
+	style_name = g_strdup ((char*) tmp);
+	xmlFree (tmp);
 
-		if (!strcmp (name, "name"))
-			style_name = val;
-		else if (!strcmp (name, "foreground"))
-			fg = val;
-		else if (!strcmp (name, "background"))
-			bg = val;
-		else if (!strcmp (name, "italic"))
-		{
-			mask |= GTK_SOURCE_STYLE_USE_ITALIC;
-			italic = str_to_bool (val);
-		}
-		else if (!strcmp (name, "bold"))
-		{
-			mask |= GTK_SOURCE_STYLE_USE_BOLD;
-			bold = str_to_bool (val);
-		}
-		else if (!strcmp (name, "underline"))
-		{
-			mask |= GTK_SOURCE_STYLE_USE_UNDERLINE;
-			underline = str_to_bool (val);
-		}
-		else if (!strcmp (name, "strikethrough"))
-		{
-			mask |= GTK_SOURCE_STYLE_USE_STRIKETHROUGH;
-			strikethrough = str_to_bool (val);
-		}
-		else if (!strcmp (name, "use-style"))
-		{
-			if (use_style != NULL)
-			{
-				g_set_error (error, ERROR_QUARK, 0,
-					     "in style '%s': duplicated use-style attribute",
-					     style_name ? style_name : "(null)");
-				gtk_source_style_free (use_style);
-				return FALSE;
-			}
-
-			use_style = gtk_source_style_scheme_get_style (scheme, val);
-
-			if (!use_style)
-			{
-				g_set_error (error, ERROR_QUARK, 0,
-					     "in style '%s': unknown style '%s'",
-					     style_name ? style_name : "(null)", val);
-				return FALSE;
-			}
-		}
-		else
+	tmp = xmlGetProp (node, BAD_CAST "use-style");
+	if (tmp != NULL)
+	{
+		if (use_style != NULL)
 		{
 			g_set_error (error, ERROR_QUARK, 0,
-				     "in style '%s': unexpected attribute '%s'",
-				     style_name ? style_name : "(null)", name);
-			gtk_source_style_free (use_style);
+				     "in style '%s': duplicated use-style attribute",
+				     style_name);
+			g_free (style_name);
+			g_object_unref (use_style);
+			return FALSE;
+		}
+
+		use_style = gtk_source_style_scheme_get_style (scheme, (char*) tmp);
+
+		if (use_style == NULL)
+		{
+			g_set_error (error, ERROR_QUARK, 0,
+				     "in style '%s': unknown style '%s'",
+				     style_name, tmp);
+			g_free (style_name);
 			return FALSE;
 		}
 	}
+	xmlFree (tmp);
 
-	if (style_name == NULL)
-	{
-		g_set_error (error, ERROR_QUARK, 0, "name attribute missing");
-		gtk_source_style_free (use_style);
-		return FALSE;
-	}
+	fg = xmlGetProp (node, BAD_CAST "foreground");
+	bg = xmlGetProp (node, BAD_CAST "background");
+	get_bool (node, "italic", &mask, GTK_SOURCE_STYLE_USE_ITALIC, &italic);
+	get_bool (node, "bold", &mask, GTK_SOURCE_STYLE_USE_BOLD, &bold);
+	get_bool (node, "underline", &mask, GTK_SOURCE_STYLE_USE_UNDERLINE, &underline);
+	get_bool (node, "strikethrough", &mask, GTK_SOURCE_STYLE_USE_STRIKETHROUGH, &strikethrough);
 
 	if (use_style)
 	{
@@ -527,7 +651,10 @@ parse_style (GtkSourceStyleScheme *scheme,
 			g_set_error (error, ERROR_QUARK, 0,
 				     "in style '%s': style attributes used along with use-style",
 				     style_name);
-			gtk_source_style_free (use_style);
+			g_object_unref (use_style);
+			g_free (style_name);
+			xmlFree (fg);
+			xmlFree (bg);
 			return FALSE;
 		}
 
@@ -536,7 +663,9 @@ parse_style (GtkSourceStyleScheme *scheme,
 	}
 	else
 	{
-		result = gtk_source_style_new (mask);
+		result = gtk_source_style_new ();
+
+		result->mask = mask;
 		result->bold = bold;
 		result->italic = italic;
 		result->underline = underline;
@@ -544,134 +673,107 @@ parse_style (GtkSourceStyleScheme *scheme,
 
 		if (fg != NULL)
 		{
-			if (gdk_color_parse (fg, &result->foreground))
-				result->mask |= GTK_SOURCE_STYLE_USE_FOREGROUND;
-			else
-				g_warning ("in style '%s': invalid color '%s'",
-					   style_name, fg);
+			result->foreground = g_intern_string ((char*) fg);
+			result->mask |= GTK_SOURCE_STYLE_USE_FOREGROUND;
 		}
 
 		if (bg != NULL)
 		{
-			if (gdk_color_parse (bg, &result->background))
-				result->mask |= GTK_SOURCE_STYLE_USE_BACKGROUND;
-			else
-				g_warning ("in style '%s': invalid color '%s'",
-					   style_name, bg);
+			result->background = g_intern_string ((char*) bg);
+			result->mask |= GTK_SOURCE_STYLE_USE_BACKGROUND;
 		}
 	}
 
 	*style_p = result;
-	*style_name_p = g_strdup (style_name);
+	*style_name_p = style_name;
+
+	xmlFree (bg);
+	xmlFree (fg);
+
 	return TRUE;
 }
 
 static void
-start_element (G_GNUC_UNUSED GMarkupParseContext *context,
-	       const gchar         *element_name,
-	       const gchar        **attribute_names,
-	       const gchar        **attribute_values,
-	       gpointer             user_data,
-	       GError             **error)
+parse_style_scheme_element (GtkSourceStyleScheme *scheme,
+			    xmlNode              *scheme_node,
+			    GError              **error)
 {
-	ParserData *data = user_data;
-	GtkSourceStyle *style;
-	gchar *style_name;
+	xmlChar *tmp;
+	xmlNode *node;
 
-	if (data->done || (data->scheme == NULL && strcmp (element_name, "style-scheme") != 0))
+	if (strcmp ((char*) scheme_node->name, "style-scheme") != 0)
 	{
 		g_set_error (error, ERROR_QUARK, 0,
 			     "unexpected element '%s'",
-			     element_name);
+			     (char*) scheme_node->name);
 		return;
 	}
 
-	if (data->scheme == NULL)
+	tmp = xmlGetProp (scheme_node, BAD_CAST "id");
+	if (tmp == NULL)
 	{
-		data->scheme = g_object_new (GTK_TYPE_SOURCE_STYLE_SCHEME, NULL);
+		g_set_error (error, ERROR_QUARK, 0, "missing 'id' attribute");
+		return;
+	}
+	scheme->priv->id = g_strdup ((char*) tmp);
+	xmlFree (tmp);
 
-		while (attribute_names != NULL && *attribute_names != NULL)
+	tmp = xmlGetProp (scheme_node, BAD_CAST "_name");
+	if (tmp != NULL)
+		scheme->priv->name = g_strdup (_((char*) tmp));
+	else if ((tmp = xmlGetProp (scheme_node, BAD_CAST "name")) != NULL)
+		scheme->priv->name = g_strdup ((char*) tmp);
+	else
+	{
+		g_set_error (error, ERROR_QUARK, 0, "missing 'name' attribute");
+		return;
+	}
+	xmlFree (tmp);
+
+	tmp = xmlGetProp (scheme_node, BAD_CAST "parent-scheme");
+	if (tmp != NULL)
+		scheme->priv->parent_id = g_strdup ((char*) tmp);
+	xmlFree (tmp);
+
+	for (node = scheme_node->children; node != NULL; node = node->next)
+	{
+		if (node->type != XML_ELEMENT_NODE)
+			continue;
+
+		if (strcmp ((char*) node->name, "style") == 0)
 		{
-			gboolean error_set = FALSE;
+			GtkSourceStyle *style;
+			gchar *style_name;
 
-			if (!strcmp (*attribute_names, "id"))
-			{
-				data->scheme->priv->id = g_strdup (*attribute_values);
-			}
-			else if (!strcmp (*attribute_names, "_name"))
-			{
-				if (data->scheme->priv->name != NULL)
-				{
-					g_set_error (error, ERROR_QUARK, 0,
-						     "duplicated name attribute");
-					error_set = TRUE;
-				}
-				else
-				{
-					data->scheme->priv->name = g_strdup (_(*attribute_values));
-				}
-			}
-			else if (!strcmp (*attribute_names, "name"))
-			{
-				if (data->scheme->priv->name != NULL)
-				{
-					g_set_error (error, ERROR_QUARK, 0,
-						     "duplicated name attribute");
-					error_set = TRUE;
-				}
-				else
-				{
-					data->scheme->priv->name = g_strdup (*attribute_values);
-				}
-			}
-			else if (!strcmp (*attribute_names, "parent-scheme"))
-			{
-				data->scheme->priv->parent_id = g_strdup (*attribute_values);
-			}
-			else
-			{
-				g_set_error (error, ERROR_QUARK, 0,
-					     "unexpected attribute '%s' in element 'style-scheme'",
-					     *attribute_names);
-				error_set = TRUE;
-			}
-
-			if (error_set)
+			if (!parse_style (scheme, node, &style_name, &style, error))
 				return;
 
-			attribute_names++;
-			attribute_values++;
+			g_hash_table_insert (scheme->priv->styles, style_name, style);
+		}
+		else if (strcmp ((char*) node->name, "author") == 0)
+		{
+			tmp = xmlNodeGetContent	(node);
+			scheme->priv->author = g_strdup ((char*) tmp);
+			xmlFree (tmp);
+		}
+		else if (strcmp ((char*) node->name, "description") == 0)
+		{
+			tmp = xmlNodeGetContent	(node);
+			scheme->priv->description = g_strdup ((char*) tmp);
+			xmlFree (tmp);
+		}
+		else if (strcmp ((char*) node->name, "_description") == 0)
+		{
+			tmp = xmlNodeGetContent	(node);
+			scheme->priv->description = g_strdup (_((char*) tmp));
+			xmlFree (tmp);
+		}
+		else
+		{
+			g_set_error (error, ERROR_QUARK, 0, "unknown node '%s'", node->name);
+			return;
 		}
-
-		return;
 	}
-
-	if (strcmp (element_name, "style") != 0)
-	{
-		g_set_error (error, ERROR_QUARK, 0,
-			     "unexpected element '%s'",
-			     element_name);
-		return;
-	}
-
-	if (!parse_style (data->scheme, attribute_names, attribute_values,
-			  &style_name, &style, error))
-	{
-		return;
-	}
-
-	g_hash_table_insert (data->scheme->priv->styles, style_name, style);
-}
-
-static void
-end_element (G_GNUC_UNUSED GMarkupParseContext *context,
-	     const gchar         *element_name,
-	     gpointer             user_data,
-	     G_GNUC_UNUSED GError **error)
-{
-	ParserData *data = user_data;
-	if (!strcmp (element_name, "style-scheme"))
-		data->done = TRUE;
 }
 
 /**
@@ -686,41 +788,63 @@ end_element (G_GNUC_UNUSED GMarkupParseContext *context,
 GtkSourceStyleScheme *
 _gtk_source_style_scheme_new_from_file (const gchar *filename)
 {
-	GMarkupParseContext *ctx = NULL;
+	char *text;
+	gsize text_len;
+	xmlDoc *doc;
+	xmlNode *node;
 	GError *error = NULL;
-	char *text = NULL;
-	ParserData data;
-	GMarkupParser parser = {start_element, end_element, NULL, NULL, NULL};
+	GtkSourceStyleScheme *scheme;
 
 	g_return_val_if_fail (filename != NULL, NULL);
 
-	if (!g_file_get_contents (filename, &text, NULL, &error))
+	if (!g_file_get_contents (filename, &text, &text_len, &error))
 	{
+		gchar *filename_utf8 = g_filename_display_name (filename);
 		g_warning ("could not load style scheme file '%s': %s",
-			   filename, error->message);
+			   filename_utf8, error->message);
+		g_free (filename_utf8);
 		g_error_free (error);
 		return NULL;
 	}
 
-	data.scheme = NULL;
-	data.done = FALSE;
+	doc = xmlParseMemory (text, text_len);
 
-	ctx = g_markup_parse_context_new (&parser, 0, &data, NULL);
-
-	if (!g_markup_parse_context_parse (ctx, text, -1, &error) ||
-	    !g_markup_parse_context_end_parse (ctx, &error))
+	if (!doc)
 	{
-		if (data.scheme != NULL)
-			g_object_unref (data.scheme);
-		data.scheme = NULL;
+		gchar *filename_utf8 = g_filename_display_name (filename);
+		g_warning ("could not parse scheme file '%s'", filename_utf8);
+		g_free (filename_utf8);
+		g_free (text);
+		return NULL;
+	}
+
+	node = xmlDocGetRootElement (doc);
+
+	if (node == NULL)
+	{
+		gchar *filename_utf8 = g_filename_display_name (filename);
+		g_warning ("could not load scheme file '%s': empty document", filename_utf8);
+		g_free (filename_utf8);
+		g_free (text);
+		return NULL;
+	}
+
+	scheme = g_object_new (GTK_TYPE_SOURCE_STYLE_SCHEME, NULL);
+	parse_style_scheme_element (scheme, node, &error);
+
+	if (error != NULL)
+	{
+		gchar *filename_utf8 = g_filename_display_name (filename);
 		g_warning ("could not load style scheme file '%s': %s",
-			   filename, error->message);
+			   filename_utf8, error->message);
+		g_free (filename_utf8);
 		g_error_free (error);
+		g_object_unref (scheme);
+		scheme = NULL;
 	}
 
-	g_markup_parse_context_free (ctx);
 	g_free (text);
-	return data.scheme;
+	return scheme;
 }
 
 /**
@@ -764,14 +888,19 @@ _gtk_source_style_scheme_set_parent (GtkSourceStyleScheme *scheme,
 }
 
 /**
- * _gtk_source_style_scheme_default_new:
+ * _gtk_source_style_scheme_get_default:
  *
- * Returns: new default style scheme. Not clear what it means though.
+ * Returns: default style scheme to be used when user didn't set
+ * style scheme explicitly.
  *
  * Since: 2.0
  */
 GtkSourceStyleScheme *
-_gtk_source_style_scheme_default_new (void)
+_gtk_source_style_scheme_get_default (void)
 {
-	return _gtk_source_style_scheme_new ("gvim", "GVim");
+	GtkSourceStyleManager *manager;
+
+	manager = gtk_source_style_manager_get_default ();
+
+	return gtk_source_style_manager_get_scheme (manager, "gvim");
 }
