@@ -46,28 +46,5 @@ gtk_source_view_smart_home_end_type_get_type (void)
 }
 
 
-/* enumerations from "gtksourcestyle.h" */
-static const GFlagsValue _gtk_source_style_mask_values[] = {
-  { GTK_SOURCE_STYLE_USE_BACKGROUND, "GTK_SOURCE_STYLE_USE_BACKGROUND", "use_background" },
-  { GTK_SOURCE_STYLE_USE_FOREGROUND, "GTK_SOURCE_STYLE_USE_FOREGROUND", "use_foreground" },
-  { GTK_SOURCE_STYLE_USE_ITALIC, "GTK_SOURCE_STYLE_USE_ITALIC", "use_italic" },
-  { GTK_SOURCE_STYLE_USE_BOLD, "GTK_SOURCE_STYLE_USE_BOLD", "use_bold" },
-  { GTK_SOURCE_STYLE_USE_UNDERLINE, "GTK_SOURCE_STYLE_USE_UNDERLINE", "use_underline" },
-  { GTK_SOURCE_STYLE_USE_STRIKETHROUGH, "GTK_SOURCE_STYLE_USE_STRIKETHROUGH", "use_strikethrough" },
-  { 0, NULL, NULL }
-};
-
-GType
-gtk_source_style_mask_get_type (void)
-{
-  static GType type = 0;
-
-  if (!type)
-    type = g_flags_register_static ("GtkSourceStyleMask", _gtk_source_style_mask_values);
-
-  return type;
-}
-
-
 /* Generated data ends here */
 
