@@ -16,12 +16,6 @@ G_BEGIN_DECLS
 GType gtk_source_view_smart_home_end_type_get_type (void);
 G_END_DECLS
 
-#include <gtksourceview/gtksourcestyle.h>
-G_BEGIN_DECLS
-#define GTK_TYPE_SOURCE_STYLE_MASK gtk_source_style_mask_get_type()
-GType gtk_source_style_mask_get_type (void);
-G_END_DECLS
-
 #endif /* __GTKSOURCEVIEW_TYPEBUILTINS_H__ */
 
 /* Generated data ends here */
